@@ -14,13 +14,17 @@ from graphql.type import (
 )
 
 
-_i_token_delimiter = r"(?:^|[\s\r\n]+|$)"
+_i_token_delimiter = r"(?:^|[\s]+|$)"
 _i_token_name = "[_A-Za-z][_0-9A-Za-z]*"
 _i_token_arguments = r"\([^)]*\)"
 _i_token_location = "[_A-Za-z][_0-9A-Za-z]*"
+_i_token_description_block_string = r"(?:\"{3}(?:[^\"]{1,}|[\s])\"{3})"
+_i_token_description_single_line = r"(?:\"(?:[^\"\n\r])*?\")"
 
 _r_directive_definition = re.compile(
     "("
+    f"{_i_token_delimiter}"
+    f"(?:{_i_token_description_block_string}|{_i_token_description_single_line})??"
     f"{_i_token_delimiter}directive"
     f"(?:{_i_token_delimiter})?@({_i_token_name})"
     f"(?:(?:{_i_token_delimiter})?{_i_token_arguments})?"
@@ -48,6 +52,11 @@ _allowed_directives = [
     "provides",  # Federation directive.
     "key",  # Federation directive.
     "extends",  # Federation directive.
+    "link",  # Federation 2 directive.
+    "shareable",  # Federation 2 directive.
+    "tag",  # Federation 2 directive.
+    "override",  # Federation 2 directive.
+    "inaccessible",  # Federation 2 directive.
 ]
 
 
@@ -62,7 +71,7 @@ def purge_schema_directives(joined_type_defs: str) -> str:
 
 
 def resolve_entities(_: Any, info: GraphQLResolveInfo, **kwargs) -> Any:
-    representations = list(kwargs.get("representations", list()))
+    representations = list(kwargs.get("representations", []))
 
     result = []
     for reference in representations:
