@@ -1,6 +1,6 @@
-import datetime
+from datetime import datetime
 from inspect import isawaitable
-from typing import Any
+from typing import Any, List, Optional, cast
 
 from graphql import GraphQLResolveInfo
 
@@ -23,22 +23,23 @@ TIMESTAMP_FORMAT = "%Y-%m-%dT%H:%M:%S.%fZ"
 
 
 class ApolloTracingExtension(Extension):
-    def __init__(self):
-        self.start_date = None
-        self.start_timestamp = None
-        self.resolvers = []
+    def __init__(self, trace_default_resolver: bool = False):
+        self.trace_default_resolver = trace_default_resolver
+        self.start_date: Optional[datetime] = None
+        self.start_timestamp: Optional[int] = None
+        self.resolvers: List[dict] = []
 
         self._totals = None
 
     def request_started(self, context: ContextValue):
-        self.start_date = datetime.datetime.utcnow()
+        self.start_date = datetime.utcnow()
         self.start_timestamp = perf_counter_ns()
 
     async def resolve(
-        self, next_: Resolver, parent: Any, info: GraphQLResolveInfo, **kwargs
+        self, next_: Resolver, obj: Any, info: GraphQLResolveInfo, **kwargs
     ):
-        if not should_trace(info):
-            result = next_(parent, info, **kwargs)
+        if not should_trace(info, self.trace_default_resolver):
+            result = next_(obj, info, **kwargs)
             if isawaitable(result):
                 result = await result
             return result
@@ -49,11 +50,11 @@ class ApolloTracingExtension(Extension):
             "parentType": str(info.parent_type),
             "fieldName": info.field_name,
             "returnType": str(info.return_type),
-            "startOffset": start_timestamp - self.start_timestamp,
+            "startOffset": start_timestamp - cast(int, self.start_timestamp),
         }
         self.resolvers.append(record)
         try:
-            result = next_(parent, info, **kwargs)
+            result = next_(obj, info, **kwargs)
             if isawaitable(result):
                 result = await result
             return result
@@ -69,7 +70,7 @@ class ApolloTracingExtension(Extension):
     def _get_totals(self):
         return {
             "start": self.start_date,
-            "end": datetime.datetime.utcnow(),
+            "end": datetime.utcnow(),
             "duration": perf_counter_ns() - self.start_timestamp,
             "resolvers": self.resolvers,
         }
@@ -90,10 +91,10 @@ class ApolloTracingExtension(Extension):
 
 class ApolloTracingExtensionSync(ApolloTracingExtension):
     def resolve(
-        self, next_: Resolver, parent: Any, info: GraphQLResolveInfo, **kwargs
+        self, next_: Resolver, obj: Any, info: GraphQLResolveInfo, **kwargs
     ):  # pylint: disable=invalid-overridden-method
         if not should_trace(info):
-            result = next_(parent, info, **kwargs)
+            result = next_(obj, info, **kwargs)
             return result
 
         start_timestamp = perf_counter_ns()
@@ -102,11 +103,11 @@ class ApolloTracingExtensionSync(ApolloTracingExtension):
             "parentType": str(info.parent_type),
             "fieldName": info.field_name,
             "returnType": str(info.return_type),
-            "startOffset": start_timestamp - self.start_timestamp,
+            "startOffset": start_timestamp - cast(int, self.start_timestamp),
         }
         self.resolvers.append(record)
         try:
-            result = next_(parent, info, **kwargs)
+            result = next_(obj, info, **kwargs)
             return result
         finally:
             end_timestamp = perf_counter_ns()
