@@ -1,16 +1,18 @@
 from inspect import isawaitable
+from dataclasses import dataclass
 from typing import (
     Any,
     AsyncGenerator,
     Callable,
+    Collection,
     List,
     Optional,
-    Sequence,
     Tuple,
     Type,
     Union,
+    TypeVar,
 )
-from typing_extensions import Protocol
+from typing_extensions import Protocol, runtime_checkable
 
 from graphql import (
     DocumentNode,
@@ -20,6 +22,10 @@ from graphql import (
     GraphQLSchema,
 )
 from graphql.validation.rules import ASTValidationRule
+from graphql.execution import Middleware
+
+from starlette.websockets import WebSocket
+
 
 # Note: this should be [Any, GraphQLResolveInfo, **kwargs],
 # but this is not achieveable with python types yet:
@@ -36,15 +42,51 @@ ContextValue = Union[Any, Callable[[Any], Any]]
 RootValue = Union[Any, Callable[[Optional[Any], DocumentNode], Any]]
 
 ValidationRules = Union[
-    Sequence[Type[ASTValidationRule]],
+    Collection[Type[ASTValidationRule]],
     Callable[
-        [Optional[Any], DocumentNode, dict], Optional[Sequence[Type[ASTValidationRule]]]
+        [Optional[Any], DocumentNode, dict],
+        Optional[Collection[Type[ASTValidationRule]]],
     ],
 ]
 
 ExtensionList = Optional[List[Union[Type["Extension"], Callable[[], "Extension"]]]]
 
 
+Extensions = Union[
+    Callable[[Any, Optional[ContextValue]], ExtensionList], ExtensionList
+]
+MiddlewareList = Optional[List[Middleware]]
+Middlewares = Union[
+    Callable[[Any, Optional[ContextValue]], MiddlewareList], MiddlewareList
+]
+
+
+@dataclass
+class Operation:
+    id: str
+    name: Optional[str]
+    generator: AsyncGenerator
+
+
+OnConnect = Callable[[WebSocket, Any], Any]
+OnDisconnect = Callable[[WebSocket], Any]
+OnOperation = Callable[[WebSocket, Operation], Any]
+OnComplete = Callable[[WebSocket, Operation], Any]
+
+
+class WebSocketConnectionError(Exception):
+    """Special error class enabling custom error reporting for on_connect"""
+
+    def __init__(self, payload: Optional[Union[dict, str]] = None):
+        if isinstance(payload, dict):
+            self.payload = payload
+        elif payload:
+            self.payload = {"message": str(payload)}
+        else:
+            self.payload = {"message": "Unexpected error has occurred."}
+
+
+@runtime_checkable
 class Extension(Protocol):
     def request_started(self, context: ContextValue):
         pass  # pragma: no cover
@@ -53,9 +95,9 @@ class Extension(Protocol):
         pass  # pragma: no cover
 
     async def resolve(
-        self, next_: Resolver, parent: Any, info: GraphQLResolveInfo, **kwargs
+        self, next_: Resolver, obj: Any, info: GraphQLResolveInfo, **kwargs
     ):
-        result = next_(parent, info, **kwargs)
+        result = next_(obj, info, **kwargs)
         if isawaitable(result):
             result = await result
         return result
@@ -69,11 +111,18 @@ class Extension(Protocol):
 
 class ExtensionSync(Extension):
     def resolve(
-        self, next_: Resolver, parent: Any, info: GraphQLResolveInfo, **kwargs
+        self, next_: Resolver, obj: Any, info: GraphQLResolveInfo, **kwargs
     ):  # pylint: disable=invalid-overridden-method
-        return next_(parent, info, **kwargs)
+        return next_(obj, info, **kwargs)
 
 
+@runtime_checkable
 class SchemaBindable(Protocol):
     def bind_to_schema(self, schema: GraphQLSchema) -> None:
         pass  # pragma: no cover
+
+
+SubscriptionHandler = TypeVar("SubscriptionHandler")
+SubscriptionHandlers = Union[
+    Tuple[Type[SubscriptionHandler]], List[Type[SubscriptionHandler]]
+]
